function y = hsv2rgb(x)

%HSV2RGB Convert hue-saturation-value colors to red-green-blue.
% x and y must be 3-D colour image arrays, covering the range 0 to 1.
%
% Simplified from the standard matlab function by 
% Nick Kingsbury, Cambridge University, 2006.

% Get HSV slices.
h = x(:,:,1); s = x(:,:,2); v = x(:,:,3); 
siz = size(h);
s = s(:); v = v(:); 
h = 6*h(:); % Scale up hue to cover the range 0 to 6.

k = fix(h-6*eps); % Measure integer and fractional parts of hue.
f = h-k;
t = 1-s;
n = 1-s.*f;
p = 1-(s.*(1-f));
e = ones(size(h));
k0 = double(k==0); % These 6 lines increase speed over the Matlab hsv2rgb().
k1 = double(k==1);
k2 = double(k==2);
k3 = double(k==3);
k4 = double(k==4);
k5 = double(k==5);
% Compute r,g,b from h and s only.
r = k0.*e + k1.*n + k2.*t + k3.*t + k4.*p + k5.*e;
g = k0.*p + k1.*e + k2.*e + k3.*n + k4.*t + k5.*t;
b = k0.*t + k1.*t + k2.*p + k3.*1 + k4.*1 + k5.*n;

% Correct r,g,b for v component using f, and assemble y.
f = v./max([r(:);g(:);b(:)]);
y = zeros([siz,3]);
y(:,:,1) = reshape(f.*r,siz);
y(:,:,2) = reshape(f.*g,siz);
y(:,:,3) = reshape(f.*b,siz);
return

