function y = hsv2rgb(x)

%HSV2RGB Convert hue-saturation-value colors to red-green-blue.
% x and y must be 3-D colour image arrays, covering the range 0 to 1.
%
% Simplified from the standard matlab function by 
% Nick Kingsbury, Cambridge University, 2006.

h = x(:,:,1); s = x(:,:,2); v = x(:,:,3); 
  siz = size(h);
  s = s(:); v = v(:); h = h(:);

h = 6*h(:);
k = fix(h-6*eps);
f = h-k;
t = 1-s;
n = 1-s.*f;
p = 1-(s.*(1-f));
e = ones(size(h));
k0 = double(k==0);
k1 = double(k==1);
k2 = double(k==2);
k3 = double(k==3);
k4 = double(k==4);
k5 = double(k==5);
r = k0.*e + k1.*n + k2.*t + k3.*t + k4.*p + k5.*e;
g = k0.*p + k1.*e + k2.*e + k3.*n + k4.*t + k5.*t;
b = k0.*t + k1.*t + k2.*p + k3.*1 + k4.*1 + k5.*n;
f = v./max([r(:);g(:);b(:)]);

if nargout<=1,
  if (threeD | nargin==3),
    rout = zeros([siz,3]);
    rout(:,:,1) = reshape(f.*r,siz);
    rout(:,:,2) = reshape(f.*g,siz);
    rout(:,:,3) = reshape(f.*b,siz);
  else
    rout = [f.*r f.*g f.*b];
  end
else
  rout = reshape(f.*r,siz);
  g = reshape(f.*g,siz);
  b = reshape(f.*b,siz);
end
