function y = im_enhance(x,alphah,alphav)
% function y = im_enhance(x,alphah,alphav)
%
% Enhance the image x using lowpass filtering with edge inhibition.
% alphah and alphav specify the horizontal and vertical coefficients 
% for the enhancement process.  
% If alpha is positive (and <= 0.25) then the image is denoised by 
% lowpass filtering in proportion to the alpha at each location. 
% If alpha is negative (and >= -0.25) then the image edges are enhanced
% by highpass filtering in proportion to -alpha at each location.
% The magnitude of the alphas should not exceed 0.25 for proper operation
% of the filters.
% 
% Nick Kingsbury, Cambridge University, 2006.

sx = size(x);
y = x;

if sum(alphav(:)) >0, wh = waitbar(0,'Denoising image');
else wh = waitbar(0,'Sharpening image');
end

% Zero row and column for padding out and offsetting the difference images
% at the image edges.
zr = zeros(1,sx(2)); 
zc = zeros(sx(1),1);

% Apply adaptive filtering, controlled by alphav and alphah, to each colour
% layer.
for k = 1:3,
    waitbar(k/4);
    xk = x(:,:,k);
    xd = diff(xk) .* alphav;
    xk = xk - [zr; xd] + [xd; zr]; % Apply vertical filtering
    xd = diff(xk.').' .* alphah;
    y(:,:,k) = xk - [zc  xd] + [xd  zc]; % Apply horizontal filtering
end

close(wh) % Close wait bar
return
