function yui = im_rotate(xui,theta)

% function yui = im_rotate(xui,theta)
% Rotate xui by theta degrees.
%
% Nick Kingsbury, Cambridge University, 2006.

% Use quick rotations if theta is a multiple of 90 deg.
if theta == 90,
    for k = 1:3, yui(:,:,k) = xui(:,end:-1:1,k)'; end
elseif theta == -90,
    for k = 1:3, yui(:,:,k) = xui(end:-1:1,:,k)'; end
elseif abs(theta) == 180,
    yui = xui(end:-1:1,end:-1:1,:);
elseif theta == 0,
    yui = xui;
else  % Slower function for any angle.
    wh = waitbar(0,'Rotating image');
    thrad = theta * pi / 180; % theta in radians
    cs = cos(thrad); 
    sn = sin(thrad);
    R = [cs sn; -sn cs];
    sx = size(xui);
    sy = ceil(sx(1:2) * abs(R)); % Size for rotated image y
    
    % p and q are vectors of distances of each row/column of y
    % from its centre.
    p = [1:sy(1)]' - (sy(1)+1)/2;
    q = [1:sy(2)] - (sy(2)+1)/2;
    
    % u and v are the matrices of the row and column indices of the
    % locations in xui from which each interpolated point in y is to come.
    u = cs*p*ones(1,sy(2)) + sn*ones(sy(1),1)*q + (sx(1)+1)/2;
    v = cs*ones(sy(1),1)*q - sn*p*ones(1,sy(2)) + (sx(2)+1)/2;
    
    % Perform the rotation using interp2().
    yui = uint8(zeros(sy(1),sy(2),3));
    for k = 1:3,
        y = interp2(double(xui(:,:,k)),v,u,'linear');
        % Replace all points coming from outside of xui by mid grey. 
        y(find(isnan(y))) = 128;
        yui(:,:,k) = uint8(y + 0.5); % Convert output image back to uint8.
        waitbar(k/3);
    end
    close(wh) % Close wait bar
end
return
