% ph_enhance.m
% Routine called by 'Enhance' menu item in photo editor.
%
% Nick Kingsbury, Cambridge University, November 2005.

% Set up figure window for commands
figure(2);
set(gcf,'position',[400  624  450  100]);
set(gcf,'numbertitle','off','name','Enhance Image');

% Initialise colour correction gains
sc = 100;
if exist('thresh') ~= 1, thresh = sc*[0.05 0.1 1.0]'; end

% Buttons for different lighting ranges.
label = strvcat('Denoise thr:','Sharpen thr:','Strength(%):');
for k = 1:3,
    ypos = 70 - 20*k; xpos = 10;
    slide(k) = uicontrol(gcf,'style','slider',...
        'sliderstep',[1 10]/100,'pos',[xpos+70 ypos 120 20]);
    uicontrol(gcf,'style','text','str',label(k,:),'pos',[xpos ypos 70 20]);
    edbox(k) = uicontrol(gcf,'style','edit','pos',[xpos+190 ypos 60 20]);
end
uicontrol(gcf,'style','text','str','Adjust Thresholds','pos',[70 70 120 16]);

% Other buttons: slider, edit box, and accept box.
denoise = uicontrol(gcf,'str','Reduce noise','pos',[340 70 100 20]);
sharpen = uicontrol(gcf,'str','Enhance edges','pos',[340 40 100 20]);
accpt = uicontrol(gcf,'str','Accept','pos',[380 10 60 20]);
revert = uicontrol(gcf,'str','Revert','pos',[310 10 60 20]);

% Define callback strings:
% Update sliders and edit boxes from light and replot yui data.

y = double(xui) + 0.5;
sx = size(xui);

cb = ['for k=1:3, set(slide(k),''value'',thresh(k)/sc);'...
        'set(edbox(k),''string'',sprintf(''%.2f'',thresh(k))); end;'...
        '[gainh,gainv] = im_edges(y,thresh(1:2)); cg = thresh(3)/400;'];

iter = 0;
cb1 = ['y = im_enhance(y,cg*gainh(:,:,1),cg*gainv(:,:,1)); iter = iter + 1;'];

cb2 = ['y = im_enhance(y,cg*(gainh(:,:,2)-1),cg*(gainv(:,:,2)-1)); iter = iter + 1;'];

cb3 = ['yui=uint8(y); update_y; figure(2);'];

eval(cb)
 
% Define callback functions for each button to set theta and perform rotation.
set(slide(1),'call',['thresh(1)=sc*get(slide(1),''value'');' cb]);
set(slide(2),'call',['thresh(2)=sc*get(slide(2),''value'');' cb]);
set(slide(3),'call',['thresh(3)=sc*get(slide(3),''value'');' cb]);
set(edbox(1),'call',['thresh(1)=sscanf(get(edbox(1),''string''),''%f'');' cb]);
set(edbox(2),'call',['thresh(2)=sscanf(get(edbox(2),''string''),''%f'');' cb]);
set(edbox(3),'call',['thresh(3)=sscanf(get(edbox(3),''string''),''%f'');' cb]);
set(denoise,'call',[cb1  cb3]);
set(sharpen,'call',[cb2  cb3]);
set(accpt,'call','xui=yui; close(2); newbefore=1; showimages;');
set(revert,'call',['y = double(xui)+0.5;' cb3]);



