% ph_lightshift.m
% Routine called by 'Lighting shift' menu item in photo editor.
%
% Nick Kingsbury, Cambridge University, 2006.

switch mode
    
    case 'Init'  
        % Set up figure window for Rotate commands
        figure;
        set(gcf,'position',[fig2pos-[0 14]  410  120]);
        set(gcf,'numbertitle','off','name','Lighting Shift');
        
        % Initialise colour correction gains
        if exist('light') ~= 1, light = [0;0;0]; end
        if exist('cgamma') ~= 1, cgamma = 1; end
        
        % Buttons for different lighting ranges.
        label = {'Shadows:','Mid tones:','Highlights:','Gamma:'};
        for k = 1:4,
          if k < 4, ypos = 95 - 20*k; xpos = 40*k - 30;
          else ypos = 10; xpos = 10;
          end
          slide(k) = uicontrol(gcf,'style','slider','sliderstep',[1 10]/100,...
            'pos',[xpos+60 ypos 120 20],'call','mode=''Slider''; ph_lightshift');
          uicontrol(gcf,'style','text','str',label{k},'pos',[xpos ypos 60 20]);
          edbox(k) = uicontrol(gcf,'style','edit','pos',[xpos+180 ypos 60 20],...
            'call','mode=''Edit box''; ph_lightshift');
        end
        uicontrol(gcf,'style','text','str','Adjust Lighting Balance',...
            'pos',[70 95 120 16]);
        
        % Other buttons: reset and close box.
        resetbtn = uicontrol(gcf,'str','Reset','pos',[340 70 60 20],...
            'call','mode=''Reset''; ph_lightshift');
        histequ = uicontrol(gcf,'str','Hist equal','pos',[340 40 60 20],...
            'call','mode=''Hist equal''; ph_lightshift');
        closebtn = uicontrol(gcf,'str','Close','pos',[340 10 60 20],...
            'call','mode=''Close''; ph_lightshift');
        mode = 'Lighting';
        
    case 'Slider'
        for k = 1:3,
            light(k) = 0.5*(get(slide(k),'value') - 0.5);
            set(edbox(k),'string',sprintf('%.3f',light(k)));
        end
        cgamma = max(2*get(slide(4),'value'),0.1);
        set(edbox(4),'string',sprintf('%.2f',cgamma));
        mode = 'Lighting';
        
    case 'Edit box'
        for k = 1:3,
            light(k) = sscanf(get(edbox(k),'string'),'%f');
            set(slide(k),'value',2*light(k) + 0.5);
        end
        cgamma = max(sscanf(get(edbox(4),'string'),'%f'),0.1);
        set(slide(4),'value',0.5*cgamma);
        mode = 'Lighting';

    case 'Reset'
        light = [0;0;0]; 
        cgamma = 1; 
        mode='Lighting';
        
    case 'Hist equal'
        yui = im_histeq(xui);
        update_y; 
        figure(2);
        
    case 'Close'
        close(2); 
        newbefore = 1; 
        showimages;
end
    
switch mode
    
    case 'Lighting'
        % Perform lighting correction.
        if all(light == 0) & cgamma == 1, yui = xui;
        else [yui,light] = im_lighting(xui,light,cgamma);
        end
        % Update sliders and edit boxes from light and replot yui data.
        for k=1:3, 
            set(slide(k),'value',2*light(k) + 0.5);
            set(edbox(k),'string',sprintf('%.3f',light(k)));
        end;
        set(slide(4),'value',0.5*cgamma);
        set(edbox(4),'string',sprintf('%.2f',cgamma));
        update_y; 
        figure(2);
end
    
