% ph_rotate.m
% Routine called by 'Rotate' menu item in photo editor.
%
% Nick Kingsbury, Cambridge University, 2006.

switch mode
    
    case 'Init'  
        % Set up figure window for Rotate commands
        figure(2);
        set(gcf,'position',[fig2pos  300  100]);
        set(gcf,'numbertitle','off','name','Rotate Image');
        
        % Initialise rotation angle theta if it does not exist.
        if exist('theta') ~= 1, theta = 0; end
        
        % Buttons for multiples of 90 deg.
        label = {'-90 deg',' 0 deg ',' 90 deg','180 deg'};
        for u = 1:4,
            btn(u) = uicontrol(gcf,'str',label{u},'pos',[60*u-30 60 60 20]);
        end
        set(btn(1),'call','theta=-90; mode=''Rotate''; ph_rotate');
        set(btn(2),'call','theta=0; mode=''Rotate''; ph_rotate');
        set(btn(3),'call','theta=90; mode=''Rotate''; ph_rotate');
        set(btn(4),'call','theta=180; mode=''Rotate''; ph_rotate');
        
        uicontrol(gcf,'style','text','str',...
            'Positive Rotation = Anti-Clockwise','pos',[30 80 240 16]);
        
        % Other buttons: slider, edit box, and close box.
        slide = uicontrol(gcf,'style','slider','value',theta/360+0.5,...
            'sliderstep',[1 10]/360,'pos',[10 10 120 20],...
            'call','mode=''Slider''; ph_rotate');
        uicontrol(gcf,'style','text','str','Adjust Rotation',...
            'pos',[10 30 120 16]);
        edbox = uicontrol(gcf,'style','edit','string',sprintf('%.2f',theta),...
            'pos',[150 10 60 20],'call','mode=''Edit box''; ph_rotate');
        uicontrol(gcf,'style','text','str','Rotation (deg)',...
            'pos',[140 30 80 16]);
        closebtn = uicontrol(gcf,'str','Close','pos',[230 10 60 20],...
            'call','mode=''Close''; ph_rotate');
         
    case 'Slider'
        theta = (get(slide,'value')-0.5)*360;
        mode = 'Rotate';
        
    case 'Edit box'
        theta = sscanf(get(edbox,'string'),'%f');
        mode = 'Rotate';
       
    case 'Close'
        close(2); 
        newbefore = 1; 
        showimages;
        
end
    
switch mode
    
    case 'Rotate'
        % Update slider position and edit box value.
        set(slide,'value',theta/360+0.5);
        set(edbox,'string',sprintf('%.2f',theta));
        % Rotate the image and display it.
        yui = im_rotate(xui,theta); 
        newbefore=0; 
        showimages
        
end
